@@ -992,6 +992,8 @@ short valid_domain (const char *dname)
         (dname[0] == '-') ||
         (dname[len - 1] == '.') ||
         (dname[0] == '.') ||
+        strstr(dname, ".-") ||
+        strstr(dname, "-.") ||
         strstr(dname, "..")) {
         return 0;
     }
@@ -1072,6 +1074,7 @@ int netcfg_get_hostname(struct debconfclient *client, char *template, char *host
 int netcfg_get_domain(struct debconfclient *client,  char domain[])
 {
     int ret;
+    char buf[6];
 
     if (have_domain == 1)
     {
@@ -1081,23 +1084,35 @@ int netcfg_get_domain(struct debconfclient *client,  char domain[])
         return 0;
     }
 
-    debconf_input (client, "high", "netcfg/get_domain");
-    ret = debconf_go(client);
+    while(1) {
+        debconf_input(client, "high", "netcfg/get_domain");
+        ret = debconf_go(client);
+        if (ret)
+            return ret;
 
-    if (ret)
-        return ret;
-
-    debconf_get (client, "netcfg/get_domain");
-
-    *domain = '\0';
-    if (!empty_str(client->value)) {
-        const char *start = client->value;
-        while (*start == '.')
-            ++start; /* trim leading dots */
-        strncpy(domain, start, MAXHOSTNAMELEN);
+        debconf_get(client, "netcfg/get_domain");
+        *domain = '\0';
+        if (empty_str(client->value))
+            return 0;
+        else {
+            const char *start = client->value;
+            while (*start == '.')
+                ++start; /* trim leading dots */
+            if (valid_domain(start)) {
+                strncpy(domain, start, MAXHOSTNAMELEN);
+                return 0;
+            }
+            di_info("%s is an invalid domain name", start);
+            debconf_subst(client, "netcfg/invalid_domain",
+                          "domain", start);
+            snprintf(buf, sizeof(buf), "%i", MAXHOSTNAMELEN);
+            debconf_subst(client, "netcfg/invalid_domain",
+                          "maxhostnamelen", buf);
+            debconf_input(client, "high", "netcfg/invalid_domain");
+            debconf_go(client);
+            debconf_set(client, "netcfg/get_domain", "");
+        }
     }
-
-    return 0;
 }
 
 void netcfg_write_loopback (void)
